==> uim-0.0.3.tar.gz: uim-0.0.3/helper/helper-applet.c <==
#ifdef HAVE_CONFIG_H
#  include <uim/config.h>
#endif  


#include <gtk/gtk.h>
#include <stdlib.h>
#include <locale.h>
#include <panel-applet.h>
#include <gnome.h>
#include <libgnome/libgnome.h>


static GtkWidget *opt_menu;
static GtkWidget *draw_win;
static GtkWidget *mode_menu;
static GtkItemFactory *mode_item_fact;
static GtkWidget *mode_button;
static GtkWidget *state_img;

/*static char hira[]={ 0xe3,0x81,0xb2,0 };
  static char hira2[]={ '/',0xe3,0x81,0xb2,0 };*/

static int uim_fd;
static GdkPixbuf *on_pxbuf, *off_pxbuf;
/*static int uim_stat;*/

static void 
display_help_dialog (BonoboUIComponent *uic, gpointer data, const gchar *verbname);
static void
display_about_dialog(BonoboUIComponent *uic, gpointer data, const gchar *verbname);

static gboolean
mode_button_pressed(GtkButton *mode_button, GdkEventButton *even, gpointer *user_data);
static void
menu_button_position(GtkMenu *menu, gint *x, gint *y, gboolean *push_in,
		     gpointer *user_data);

static void
activate_event(GtkWidget *g, gpointer p);

static const BonoboUIVerb uim_menu_verbs [] = {
  /* BONOBO_UI_VERB ("UimProperties", display_properties_dialog),*/
  BONOBO_UI_VERB ("UimHelp",       display_help_dialog),
  BONOBO_UI_VERB ("UimAbout",      display_about_dialog),
  BONOBO_UI_VERB_END
};


static const char uim_menu_xml [] =
        "<popup name=\"button3\">\n"
        "   <menuitem name=\"Properties Item\" verb=\"UimProperties\" _label=\"Properties ...\"\n"
        "             pixtype=\"stock\" pixname=\"gtk-properties\"/>\n"
        "   <menuitem name=\"Help Item\" verb=\"UimHelp\" _label=\"Help\"\n"
        "             pixtype=\"stock\" pixname=\"gtk-help\"/>\n"
        "   <menuitem name=\"About Item\" verb=\"UimAbout\" _label=\"About ...\"\n"
        "             pixtype=\"stock\" pixname=\"gnome-stock-about\"/>\n"
        "</popup>\n";


GtkItemFactoryEntry mode_menu_items[]={
  {("/dummy1"), NULL, G_CALLBACK(activate_event), 0, "" },
  {("/dummy2"), NULL, G_CALLBACK(activate_event), 1, "" },
  };

/* Opens gnome help application
 */
static void 
display_help_dialog (BonoboUIComponent *uic, gpointer data, const gchar *verbname)
{
  GError *error = NULL;
  
  gnome_help_display("Uim", NULL, &error);

  if (error)
    {
      GtkWidget *dialog;
      dialog = gtk_message_dialog_new(NULL, 
				      GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
				      GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
				      error->message);
      gtk_dialog_run (GTK_DIALOG (dialog));
      gtk_widget_destroy (dialog);
      g_error_free (error);
      error = NULL;
    }
}

/* Just the about window... If it's already open, just focus it
 */
static void
display_about_dialog(BonoboUIComponent *uic, gpointer data, const gchar *verbname)
{
	GdkPixbuf *icon;
	GtkWidget *dialog;
	const char *authors[] = {"Yusuke TABATA <yusuke@cherubim.icw.co.jp> \nTOKUNAGA Hiroyuki <tkng@xem.jp>",NULL};
		
	/* Feel free to put your names here translators :-) */
	char *translators = _("TRANSLATORS");

	//	icon = gdk_pixbuf_new_from_xpm_data(ICON_APPLET);
	
	dialog = GTK_DIALOG(gnome_about_new (_("Uim Helper Applet"), VERSION, "Copyright 2003 TOKUNAGA Hiroyuki",
					     _("Applet for indicate uim's status"),
					     (const char **) authors, NULL, 
					     strcmp("TRANSLATORS", translators) ? translators : NULL, 
					     NULL));
	
	gtk_widget_show(GTK_WIDGET(dialog));
}

static void
activate_event(GtkWidget *g, gpointer p)
{
  /*˥˥塼ͭ줿ν񤯡
    餯ݥ󥿤˳ĤƤʸ
  

      if( (int)p == 0 ) {
	gtk_button_set_label(mode_button,"a");
      }
      else if( (int)p == 1 ) {
	gtk_button_set_label(mode_button, hira);
	}*/
}


/*static gint toolbar_expose(GtkWidget *w, GdkEventExpose *e)
{
  if (uim_stat) {
    gtk_image_set_from_pixbuf(state_img, on_pxbuf);
  } else {
    gtk_image_set_from_pixbuf(state_img, off_pxbuf);
  }
  return FALSE;
}
static void handler(struct comm_event *ev)
{

  if (ev->type == COMM_EVENT_MODE) {

    gtk_option_menu_set_history(GTK_OPTION_MENU(opt_menu), ev->nr);

      if( ev->nr == 0 ) {
	gtk_button_set_label(mode_button,"a");
      }
      else if( ev->nr == 1 ) {
	gtk_button_set_label(mode_button,hira);
      }

  }
  if (ev->type == COMM_EVENT_STAT) {
    uim_stat = ev->nr;
    toolbar_expose(draw_win, NULL);
  }
}
*/
static void fd_handler(gpointer p, gint fd,
		       GdkInputCondition c)
{
  //  proc_comm(handler);
}



gchar *translate(const gchar *path, gpointer data)
{
	return gettext(path);
}


static GtkWidget * init_toolbar(void)
{
  GtkWidget *hbox;

  hbox = gtk_hbox_new(FALSE, 0);

  uim_helper_client_get_prop_list();

  mode_item_fact = gtk_item_factory_new(GTK_TYPE_MENU, "<main>", NULL);

    gtk_item_factory_set_translate_func(mode_item_fact, translate,
				      NULL, NULL);
 
  gtk_item_factory_create_items (mode_item_fact, G_N_ELEMENTS (mode_menu_items),
				 mode_menu_items, NULL); 
  
  mode_button = gtk_button_new_with_label ("a");
  gtk_button_set_relief(GTK_BUTTON(mode_button), GTK_RELIEF_HALF);
  gtk_box_pack_start(GTK_BOX(hbox), mode_button, FALSE, FALSE, 0);

  g_signal_connect(G_OBJECT(mode_button), "button_press_event",
		   G_CALLBACK(mode_button_pressed), mode_button);
  /*
  on_pxbuf = gdk_pixbuf_new_from_xpm_data(on_xpm);
  off_pxbuf = gdk_pixbuf_new_from_xpm_data(off_xpm);

  state_img = gtk_image_new_from_pixbuf(off_pxbuf);

  gtk_box_pack_start(GTK_BOX(hbox), state_img, TRUE, TRUE, 2);
  */
  return hbox;
  
}



static gboolean
mode_button_pressed(GtkButton *mode_button, GdkEventButton *event,
		    gpointer *user_data)
{
  int x,y;

  if(!event)
    return FALSE;
  else if( event->button == 3 || event->button == 2 )
    return FALSE;

  menu_button_position(NULL,&x,&y,NULL,mode_button);
  gtk_item_factory_popup(mode_item_fact, x, y, NULL, NULL);

}

static void
menu_button_position(GtkMenu *menu, gint *x, gint *y, gboolean *push_in,
			  gpointer *user_data)
{
        GtkWidget *widget;
        gint wheight;
        gint wx, wy;

	g_return_if_fail(x && y);
 	g_return_if_fail(GTK_IS_BUTTON(user_data));

	widget = GTK_WIDGET(user_data);

	gdk_window_get_origin(widget->window, x, y);

        wheight = widget->requisition.height;
	wx = widget->allocation.x;
        wy = widget->allocation.y;
         
        *y = *y + wy + wheight;
        *x = *x + wx;
}

static gboolean
uim_applet_new(PanelApplet *applet,
		 const gchar *iid,
		 gpointer     data)
{
  GtkWidget *toolbar;
 
  if (strcmp (iid, "OAFIID:GNOME_UimApplet") != 0)
    return FALSE;
  
  char *path = uim_helper_get_pathname();
  uim_fd     = uim_helper_init_client_fd(path);
  
  toolbar = init_toolbar();
 
  if(uim_fd > 0)
    gdk_input_add(uim_fd, (GdkInputCondition)GDK_INPUT_READ,
		  fd_handler, 0);

  gtk_container_add (GTK_CONTAINER (applet), toolbar);  
  gtk_widget_show_all (GTK_WIDGET (applet));

  panel_applet_setup_menu(applet,
			  uim_menu_xml,
			  uim_menu_verbs,
			  toolbar);

  return TRUE;
}



PANEL_APPLET_BONOBO_FACTORY ("OAFIID:GNOME_UimApplet_Factory",
                             PANEL_TYPE_APPLET,
                             "uim gnome2 Applet",
                             "0",
                             (PanelAppletFactoryCallback)uim_applet_new,
                             NULL);

==> uim-0.0.7.tar.gz: uim-0.0.7/helper/helper-applet.c <==
#ifdef HAVE_CONFIG_H
#  include <uim/config.h>
#endif

#include <gtk/gtk.h>
#include <stdlib.h>
#include <locale.h>
#include <panel-applet.h>
#include <gnome.h>
#include <libgnome/libgnome.h>


static GtkWidget *opt_menu;
static GtkWidget *draw_win;
static GtkWidget *mode_menu;
static GtkItemFactory *mode_item_fact;
static GtkWidget *mode_button;
static GtkWidget *state_img;
PanelApplet *uimapplet;

static int uim_fd;
static GdkPixbuf *on_pxbuf, *off_pxbuf;
/*static int uim_stat;*/

static void 
display_help_dialog (BonoboUIComponent *uic, gpointer data, const gchar *verbname);
static void
display_about_dialog(BonoboUIComponent *uic, gpointer data, const gchar *verbname);

static gboolean
mode_button_pressed(GtkButton *mode_button, GdkEventButton *even, gpointer *user_data);
static void
menu_button_position(GtkMenu *menu, gint *x, gint *y, gboolean *push_in,
		     gpointer *user_data);

static void
activate_event(GtkWidget *g, gpointer p);

static const BonoboUIVerb uim_menu_verbs [] = {
  /* BONOBO_UI_VERB ("UimProperties", display_properties_dialog),*/
  BONOBO_UI_VERB ("UimHelp",       display_help_dialog),
  BONOBO_UI_VERB ("UimAbout",      display_about_dialog),
  BONOBO_UI_VERB_END
};


static const char uim_menu_xml [] =
        "<popup name=\"button3\">\n"
        "   <menuitem name=\"Properties Item\" verb=\"UimProperties\" _label=\"Properties ...\"\n"
        "             pixtype=\"stock\" pixname=\"gtk-properties\"/>\n"
        "   <menuitem name=\"Help Item\" verb=\"UimHelp\" _label=\"Help\"\n"
        "             pixtype=\"stock\" pixname=\"gtk-help\"/>\n"
        "   <menuitem name=\"About Item\" verb=\"UimAbout\" _label=\"About ...\"\n"
        "             pixtype=\"stock\" pixname=\"gnome-stock-about\"/>\n"
        "</popup>\n";


GtkItemFactoryEntry mode_menu_items[]={
  {("/dummy1"), NULL, G_CALLBACK(activate_event), 0, "" },
  {("/dummy2"), NULL, G_CALLBACK(activate_event), 1, "" },
  };

/* Opens gnome help application
 */
static void 
display_help_dialog (BonoboUIComponent *uic, gpointer data, const gchar *verbname)
{
  GError *error = NULL;
  
  gnome_help_display("Uim", NULL, &error);

  if (error)
    {
      GtkWidget *dialog;
      dialog = gtk_message_dialog_new(NULL, 
				      GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
				      GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
				      error->message);
      gtk_dialog_run (GTK_DIALOG (dialog));
      gtk_widget_destroy (dialog);
      g_error_free (error);
      error = NULL;
    }
}

/* Just the about window... If it's already open, just focus it
 */
static void
display_about_dialog(BonoboUIComponent *uic, gpointer data, const gchar *verbname)
{
	GdkPixbuf *icon;
	GtkWidget *dialog;
	const char *authors[] = {"Yusuke TABATA <yusuke@cherubim.icw.co.jp> \nTOKUNAGA Hiroyuki <tkng@xem.jp>",NULL};
		
	/* Feel free to put your names here translators :-) */
	char *translators = _("TRANSLATORS");

	//	icon = gdk_pixbuf_new_from_xpm_data(ICON_APPLET);
	
	dialog = GTK_DIALOG(gnome_about_new (_("Uim Helper Applet"), VERSION, "Copyright 2003 TOKUNAGA Hiroyuki",
					     _("Applet for indicate uim's status"),
					     (const char **) authors, NULL, 
					     strcmp("TRANSLATORS", translators) ? translators : NULL, 
					     NULL));
	
	gtk_widget_show(GTK_WIDGET(dialog));
}

static void
activate_event(GtkWidget *g, gpointer p)
{
  /*˥˥塼ͭ줿ν񤯡
    餯ݥ󥿤˳ĤƤʸ
  

      if( (int)p == 0 ) {
	gtk_button_set_label(mode_button,"a");
      }
      else if( (int)p == 1 ) {
	gtk_button_set_label(mode_button, hira);
	}*/
}


/*static gint toolbar_expose(GtkWidget *w, GdkEventExpose *e)
{
  if (uim_stat) {
    gtk_image_set_from_pixbuf(state_img, on_pxbuf);
  } else {
    gtk_image_set_from_pixbuf(state_img, off_pxbuf);
  }
  return FALSE;
}
static void handler(struct comm_event *ev)
{

  if (ev->type == COMM_EVENT_MODE) {

    gtk_option_menu_set_history(GTK_OPTION_MENU(opt_menu), ev->nr);

      if( ev->nr == 0 ) {
	gtk_button_set_label(mode_button,"a");
      }
      else if( ev->nr == 1 ) {
	gtk_button_set_label(mode_button,hira);
      }

  }
  if (ev->type == COMM_EVENT_STAT) {
    uim_stat = ev->nr;
    toolbar_expose(draw_win, NULL);
  }
}
*/
static void fd_handler(gpointer p, gint fd,
		       GdkInputCondition c)
{
  //  proc_comm(handler);
}



gchar *translate(const gchar *path, gpointer data)
{
	return gettext(path);
}


static GtkWidget * init_toolbar(void)
{
  GtkWidget *hbox;

  hbox = gtk_hbox_new(FALSE, 0);

  uim_helper_client_get_prop_list();

  mode_item_fact = gtk_item_factory_new(GTK_TYPE_MENU, "<main>", NULL);

    gtk_item_factory_set_translate_func(mode_item_fact, translate,
				      NULL, NULL);
 
  gtk_item_factory_create_items (mode_item_fact, G_N_ELEMENTS (mode_menu_items),
				 mode_menu_items, NULL); 
  
  mode_button = gtk_button_new_with_label ("a");
  gtk_button_set_relief(GTK_BUTTON(mode_button), GTK_RELIEF_HALF);
  gtk_box_pack_start(GTK_BOX(hbox), mode_button, FALSE, FALSE, 0);

  g_signal_connect(G_OBJECT(mode_button), "button_press_event",
		   G_CALLBACK(mode_button_pressed), mode_button);
  /*
  on_pxbuf = gdk_pixbuf_new_from_xpm_data(on_xpm);
  off_pxbuf = gdk_pixbuf_new_from_xpm_data(off_xpm);

  state_img = gtk_image_new_from_pixbuf(off_pxbuf);

  gtk_box_pack_start(GTK_BOX(hbox), state_img, TRUE, TRUE, 2);
  */
  return hbox;
  
}



static gboolean
mode_button_pressed(GtkButton *mode_button, GdkEventButton *event,
		    gpointer *user_data)
{
  int x,y;

  if(!event) {
    return FALSE;
  } else if(event->button == 2 || event->button == 3) {
    gtk_propagate_event(GTK_WIDGET(uimapplet), (GdkEvent *) event);
    return FALSE;
  }

  menu_button_position(NULL,&x,&y,NULL,mode_button);
  gtk_item_factory_popup(mode_item_fact, x, y, NULL, NULL);

}

static void
menu_button_position(GtkMenu *menu, gint *x, gint *y, gboolean *push_in,
			  gpointer *user_data)
{
        GtkWidget *widget;
        gint wheight;
        gint wx, wy;

	g_return_if_fail(x && y);
 	g_return_if_fail(GTK_IS_BUTTON(user_data));

	widget = GTK_WIDGET(user_data);

	gdk_window_get_origin(widget->window, x, y);

        wheight = widget->requisition.height;
	wx = widget->allocation.x;
        wy = widget->allocation.y;
         
        *y = *y + wy + wheight;
        *x = *x + wx;
}

static gboolean
uim_applet_new(PanelApplet *applet,
		 const gchar *iid,
		 gpointer     data)
{
  GtkWidget *toolbar;
  uimapplet = applet;
 
  if (strcmp (iid, "OAFIID:GNOME_UimApplet") != 0)
    return FALSE;
  
  char *path = uim_helper_get_pathname();
  uim_fd     = uim_helper_init_client_fd(path);
  
  toolbar = init_toolbar();
 
  if(uim_fd > 0)
    gdk_input_add(uim_fd, (GdkInputCondition)GDK_INPUT_READ,
		  fd_handler, 0);

  gtk_container_add (GTK_CONTAINER (applet), toolbar);  
  gtk_widget_show_all (GTK_WIDGET (applet));

  panel_applet_setup_menu(applet,
			  uim_menu_xml,
			  uim_menu_verbs,
			  toolbar);

  return TRUE;
}



PANEL_APPLET_BONOBO_FACTORY ("OAFIID:GNOME_UimApplet_Factory",
                             PANEL_TYPE_APPLET,
                             "uim gnome2 Applet",
                             "0",
                             (PanelAppletFactoryCallback)uim_applet_new,
                             NULL);
